/*
 * MSP430_SWI2C_Master.c
 */

//******************************************************************************
//  MSP430F20xx - I2C Master Transmitter and Receiver via CPU Software (GPIO)
//
//  Description: This code library configures the MSP430 as an I2C master device
//  capable of transmitting and receiving bytes using GPIO pins.  Specific I/O
//  pins can be selected in the corresponding header file.  By default, the same
//  pins that are used by the USI module are selected.
//
//                  Master
//                  MSP430
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |                 |
//            |                 |
//            |         SDA/P2.7|-------> [I2C SLAVE SDA]
//            |         SCL/P2.6|-------> [I2C SLAVE SCL]
//
//  Note: Internal pull-ups are NOT used for SDA & SCL [DISABLED]
//
//  R. Wu
//  Texas Instruments Inc.
//  March 2008
//
// angepasst an MSP430G2553, LSM9DS0
//	16.08.2012-15.10.2014 DL9MWE
//******************************************************************************

#include "main.h"


// Set up start condition for I2C
void vMSP430_SWI2CMST_start(void)
{
	PxDIR |= SDA;		// Set to output, data low [SDA = 0]
	// Start condition hold time
#ifdef I2C_fastmode
	_NOP();				// Quick delay > 0,6us
#else
	_delay_us(4);
#endif
	PxDIR |= SCL;		// Set to output, clock low [SCL = 0]
	_NOP();
}

// Master sendet ein Byte an den I2C-Slave
void vMSP430_SWI2CMST_txByte(unsigned char data)
{
  unsigned char bits, temp;

	temp = data;
	bits = 0x08;				// Load I2C bit counter
	while (bits != 0x00) {		// Loop until all bits are shifted
		if (temp & BIT7)		// Test data bit
			PxDIR &= ~SDA;		// Set to input, SDA = 1 via pull-up
		else
			PxDIR |= SDA;		// Set to output, data low [SDA = 0]
		// SDA setup time > 100ns bzw. 250ns
		_NOP();
#if (F_MHZ == 16)
		_NOP();
		_NOP();
#endif
		PxDIR &= ~SCL;			// Set to input, clock high [SCL = 1]
		temp = (temp << 1);		// Shift bits 1 place to the left
		// Clock high time
#ifdef I2C_fastmode
//		_NOP();
		_NOP();					// Quick delay > 0,6us
#else
		_delay_us(4);
#endif
		PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
		bits = (bits - 1);		// Loop until 8 bits are sent
	}
	PxDIR &= ~SDA;				// Set to input, Slave setzt SDA
}

// set I2C_clock_pulse
void vMSP430_SWI2C_clockpulse(void) {
	PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
	// clock high time
#ifdef I2C_fastmode
	_NOP();					// ca. 0,6us delay to meet I2C spec
#else
	_delay_us(4);			// ca. 4us delay to meet I2C spec
#endif
	PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
	_NOP();
}

// Send Master Acknowledge
void vMSP430_SWI2CMST_ack(void)
{
	PxDIR |= SDA;			// Set to output, data low [SDA = 0]
	vMSP430_SWI2C_clockpulse();
	PxDIR &= ~SDA;			// Set SDA to input
}

// Send negative acknowledge
void vMSP430_SWI2CMST_nack(void)
{
	PxDIR &= ~SDA;			// Set to input, SDA = 1 via pull-up
	vMSP430_SWI2C_clockpulse();
}

// Get Slave Acknowledge
unsigned int uiMSP430_SWI2CSLV_ack(void)
{
  unsigned int error;
   PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
   _delay_us(4);			// delay 4us
   if (PxIN & SDA)
	   error = 1;
   else
	   error = 0;
   PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
   _NOP();
   return error;
}

// liest ein Byte via I2C ein
unsigned char ucMSP430_SWI2CMST_rxByte(void)  // Read 8 bits of I2C data
{
	unsigned char bits, temp = 0;	// I2C bit counter
	bits = 0x08;					// Load I2C bit counter
	while (bits > 0)				// Loop until all bits are read
	{
		PxDIR &= ~SCL;            // Set to input, SCL = 1 via pull-up
#ifdef I2C_fastmode
		_NOP();
#else
		_delay_us(4);
#endif
		temp = (temp << 1);       // Shift bits 1 place to the left
		if (PxIN & SDA)           // Check digital input
			temp = (temp + 1);      // If input is 'H' store a '1'
		PxDIR |= SCL;             // Set to output, data low [SCL = 0]
		bits = (bits - 1);        // Decrement I2C bit counter
		_NOP();
#if (F_MHZ == 16)
		_NOP();
		_NOP();
#endif
	}
	return (temp);               // Return 8-bit data byte
}

// Send I2C stop command
void vMSP430_SWI2CMST_stop(void) {
	PxDIR |= SDA;                 // Set to output, data low [SDA = 0]
	_NOP();
	_NOP();                       // Quick delay
#if (F_MHZ == 16)
    _NOP();
    _NOP();
#endif
    PxDIR &= ~SCL;                // Set to input, SCL = 1 via pull-up
#ifdef I2C_fastmode
    _NOP();
    _NOP();                       // Quick delay
#else
    _delay_us(4);
#endif
    PxDIR &= ~SDA;                // Set to input, SDA = 1 via pull-up
}


// adressiert das I2C-Device und setzt eine Register-adresse
void vI2C_set_register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
	vMSP430_SWI2CMST_start();						// Send I2C Start Condition
	vMSP430_SWI2CMST_txByte(I2C_device_address);	// [ADDR] + R/W bit = 0
	*error_code = uiMSP430_SWI2CSLV_ack();			// get slave acknowledge (ACKS)
	vMSP430_SWI2CMST_txByte(I2C_register_address);	// send ADXL345B Register Address
	*error_code += uiMSP430_SWI2CSLV_ack();			// get ACKS
}

// unsigned char aus I2C-Device-Register lesen
unsigned char ucI2C_read_char(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
  unsigned char data;
	vI2C_set_register(I2C_device_address, I2C_register_address, error_code);
	vMSP430_SWI2CMST_stop();
	vMSP430_SWI2CMST_start();						// repeated start condition
	vMSP430_SWI2CMST_txByte(I2C_device_address + 1);	// [ADDR] + R/W bit = 1
	*error_code = uiMSP430_SWI2CSLV_ack();			// get ACKS
	data = ucMSP430_SWI2CMST_rxByte();
	return data;
}


// *****************************************************************************
// ************************************ API ************************************
void vI2C_init(void)
{
  PxSEL &= ~(SCL | SDA);                    // Set GPIO function
  PxDIR |= (SCL | SDA);                     // Set output direction
  PxOUT &= ~(SCL | SDA);                    // SCL & SDA = 0 => low when = outputs
  PxDIR &= ~(SCL | SDA);                    // set to input, SCL & SDA = 1
}

unsigned char ucI2C_read_8Bit(unsigned char I2C_device_address, unsigned int* error_code) {
  unsigned char data;
	vMSP430_SWI2CMST_start();						// Send I2C Start Condition
	vMSP430_SWI2CMST_txByte(I2C_device_address+1);	// [ADDR] + R/W bit = 1
	*error_code = uiMSP430_SWI2CSLV_ack();			// get slave acknowledge (ACKS)
	data = ucMSP430_SWI2CMST_rxByte();
	vMSP430_SWI2CMST_ack();
	vMSP430_SWI2CMST_stop();
	return data;
}
void I2C_write_8Bit(unsigned char I2C_device_address, unsigned char value, unsigned int* error_code) {
	vMSP430_SWI2CMST_start();						// Send I2C Start Condition
	vMSP430_SWI2CMST_txByte(I2C_device_address);		// [ADDR] + R/W bit = 0
	*error_code = uiMSP430_SWI2CSLV_ack();			// get slave acknowledge (ACKS)
	vMSP430_SWI2CMST_txByte(value);
	*error_code += uiMSP430_SWI2CSLV_ack();			// get ACKS
	vMSP430_SWI2CMST_stop();
}

// schreibt ein Byte in ein Register des I2C-Devices
unsigned int uiI2C_write_register_value(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned char register_value) {
	unsigned int error_code;
	vI2C_set_register(I2C_device_address, I2C_register_address, &error_code);
	vMSP430_SWI2CMST_txByte(register_value); 		// write register value
	error_code += uiMSP430_SWI2CSLV_ack();			// get ACKS
	vMSP430_SWI2CMST_stop();
	return error_code;
}
// liest ein Byte aus einem Register des I2C-Devices aus
unsigned char ucI2C_read_8Bit_Register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int *error_code) {
  unsigned char data;
	data = ucI2C_read_char(I2C_device_address, I2C_register_address, error_code);
	vMSP430_SWI2CMST_nack();
	vMSP430_SWI2CMST_stop();
	return data;
}
// liest mehrere (min. 2) Bytes aus aufeinanderfolgenden Registern des I2C-Devices aus
unsigned int uiI2C_read_multiple_chars(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int numBytes, unsigned char* rxData) {
	unsigned int error_code;
	unsigned int index=1;

	rxData[0] = ucI2C_read_char(I2C_device_address, I2C_register_address, &error_code);
	vMSP430_SWI2CMST_ack();
	while(1) {
		rxData[index] = ucMSP430_SWI2CMST_rxByte();
		if(++index < numBytes)	vMSP430_SWI2CMST_ack();
		else {
			vMSP430_SWI2CMST_nack();
			break;
		}
	}
	vMSP430_SWI2CMST_stop();
	return error_code;
}

unsigned int uiI2C_write_multiple_chars(unsigned char I2C_device_address, unsigned int numBytes, unsigned char *txData) {
	unsigned int error_code, index;
	vMSP430_SWI2CMST_start();						// Send I2C Start Condition
	vMSP430_SWI2CMST_txByte(I2C_device_address);	// [ADDR] + R/W bit = 0
	error_code = uiMSP430_SWI2CSLV_ack();			// get slave acknowledge (ACKS)
	for(index=0; index < numBytes; ++index) {
		if(error_code == 0) {
			vMSP430_SWI2CMST_txByte(txData[index]);
			error_code = uiMSP430_SWI2CSLV_ack();		// get ACKS
		}
	}
	vMSP430_SWI2CMST_stop();
	return error_code;
}

// Ende I2C.c
